from alinea.adel import symbol
from alinea.adel.io import io
from alinea.adel.fit import fit

import numpy as np

import parametrisation as par

def leaves(leaves_database, points):
    """ Create leaf shapes from a measurement database . 

    """
    db, = io.load_leaf_data(leaves_database)
    leaves, = fit.fit_leaves(db, points)
    symbols = symbol.build_symbols(leaves, 0, relative_angle=False)
    return symbols


def mais_architecture(total_area = 10000, total_height = 200, 
		     pseudostem_height = 20,
                     nb_phy = 16,nb_young_phy = 6,
                     # leaf area dist function
                     lad_skew = 5,lad_rmax = 0.67,
                     # proression of distances between leaves
                     pseudostem_dist = 1.4, stem_dist = 1.,
                     # Phyllotaxy for young and mature phytomer
                     phyllotactic_angle = 180,
                     # insertion angle
                     basal_insertion = 50,
                     top_insertion = 30,
                     diam_base = 2.5,
                     diam_top = 1,
                     leaf_width_ratio = 0.1,
                     shape_factors = 0.75,hydric_stress=0., 
                     ):
    """ generate simpleMais dictionary from parameter """

    hydric_stress = float(min(99.,hydric_stress))
    stress = max(0.,min(1.,hydric_stress/100.))
    total_area -= total_area*stress*.8
    total_height -= total_height*stress*.8
    pseudostem_height -= pseudostem_height*stress*.8
    leaf_width_ratio -= leaf_width_ratio*stress*.8 

    bi = basal_insertion - (basal_insertion)*stress
    ti = top_insertion - (top_insertion)*stress
    basal_insertion = basal_insertion if basal_insertion < 10 else bi
    top_insertion = top_insertion if top_insertion < 10 else ti
    
    return par.simpleMais_param(total_area=total_area, 
		     total_height=total_height, 
		     pseudostem_height=pseudostem_height,
                     nb_phy=nb_phy,nb_young_phy = nb_young_phy,
                     # leaf area dist function
                     lad_skew=lad_skew, lad_rmax=lad_rmax,
                     # proression of distances between leaves
                     pseudostem_dist=pseudostem_dist, stem_dist=stem_dist,
                     # Phyllotaxy for young and mature phytomer
                     phyllotactic_angle=phyllotactic_angle,
                     # insertion angle
                     basal_insertion=basal_insertion,
                     top_insertion = top_insertion,
                     diam_base = diam_base,
                     diam_top = diam_top,
                     leaf_width_ratio = leaf_width_ratio,
                     shape_factors=shape_factors ,
                     )
